@@ -27,6 +27,8 @@
 #include <jni_utility.h>
 #include <jni_runtime.h>
 
+#include "binding_utility.h"
+
 using namespace KJS::Bindings;
 
 JavaClass::JavaClass (jobject anInstance)
@@ -49,10 +51,10 @@ JavaClass::JavaClass (jobject anInstance)
     // Get the fields
     jarray fields = (jarray)callJNIObjectMethod (aClass, "getFields", "()[Ljava/lang/reflect/Field;");
     long numFields = env->GetArrayLength (fields);    
-    _fields = CFDictionaryCreateMutable(NULL, numFields, &kCFTypeDictionaryKeyCallBacks, NULL);
+    _fields = CFDictionaryCreateMutable(NULL, numFields, &kCFTypeDictionaryKeyCallBacks, &FieldDictionaryValueCallBacks);
     for (i = 0; i < numFields; i++) {
         jobject aJField = env->GetObjectArrayElement ((jobjectArray)fields, i);
-        Field *aField = new JavaField (env, aJField);
+        Field *aField = new JavaField (env, aJField); // deleted when the dictionary is destroyed
         CFStringRef fieldName = CFStringCreateWithCString(NULL, aField->name(), kCFStringEncodingASCII);
         CFDictionaryAddValue ((CFMutableDictionaryRef)_fields, fieldName, aField);
         CFRelease (fieldName);
@@ -62,10 +64,10 @@ JavaClass::JavaClass (jobject anInstance)
     // Get the methods
     jarray methods = (jarray)callJNIObjectMethod (aClass, "getMethods", "()[Ljava/lang/reflect/Method;");
     long numMethods = env->GetArrayLength (methods);    
-    _methods = CFDictionaryCreateMutable(NULL, numMethods, &kCFTypeDictionaryKeyCallBacks, NULL);
+    _methods = CFDictionaryCreateMutable(NULL, numMethods, &kCFTypeDictionaryKeyCallBacks, &MethodListDictionaryValueCallBacks);
     for (i = 0; i < numMethods; i++) {
         jobject aJMethod = env->GetObjectArrayElement ((jobjectArray)methods, i);
-        Method *aMethod = new JavaMethod (env, aJMethod);
+        Method *aMethod = new JavaMethod (env, aJMethod); // deleted when the dictionary is destroyed
         CFStringRef methodName = CFStringCreateWithCString(NULL, aMethod->name(), kCFStringEncodingASCII);
         MethodList *methodList = (MethodList *)CFDictionaryGetValue ((CFMutableDictionaryRef)_methods, methodName);
         if (!methodList) {
