@@ -42,8 +42,8 @@ public:
     virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
     virtual void put(ExecState *exec, unsigned propertyName, const Value &value, int attr = None);
     
-    virtual bool hasProperty(ExecState *exec, const Identifier &propertyName) const;
-    virtual bool hasProperty(ExecState *exec, unsigned propertyName) const;
+    virtual bool hasOwnProperty(ExecState *exec, const Identifier &propertyName) const;
+    virtual bool hasOwnProperty(ExecState *exec, unsigned propertyName) const;
     virtual bool deleteProperty(ExecState *exec, const Identifier &propertyName);
     virtual bool deleteProperty(ExecState *exec, unsigned propertyName);
     
@@ -53,8 +53,9 @@ public:
     
     Bindings::Array *getConcreteArray() const { return _array; }
 
-private:
     static const ClassInfo info;
+
+private:
     Bindings::Array *_array;
 };
     
