@@ -28,6 +28,7 @@
 #include <JavaScriptCore/interpreter.h>
 #include <JavaScriptCore/object.h>
 #include <JavaScriptCore/jni_jsobject.h>
+#include <JavaScriptCore/protect.h>
 
 namespace KJS {
 
@@ -38,9 +39,9 @@ class RootObject;
 typedef RootObject *(*FindRootObjectForNativeHandleFunctionPtr)(void *);
 
 extern CFMutableDictionaryRef findReferenceDictionary(ObjectImp *imp);
-extern const Bindings::RootObject *rootForImp (ObjectImp *imp);
-extern const Bindings::RootObject *KJS::Bindings::rootForInterpreter (KJS::Interpreter *interpreter);
-extern void addNativeReference (const Bindings::RootObject *root, ObjectImp *imp);
+extern const RootObject *rootForImp (ObjectImp *imp);
+extern const RootObject *rootForInterpreter (Interpreter *interpreter);
+extern void addNativeReference (const RootObject *root, ObjectImp *imp);
 extern void removeNativeReference (ObjectImp *imp);
 
 class RootObject
@@ -53,25 +54,28 @@ public:
         _imp->deref();
 #endif
 #if USE_CONSERVATIVE_GC | TEST_CONSERVATIVE_GC
-	gcUnprotect(_imp);
+        InterpreterLock lock;
+        gcUnprotect(_imp);
 #endif
     }
     
-    void setRootObjectImp (KJS::ObjectImp *i) { 
+    void setRootObjectImp (ObjectImp *i) { 
+#if USE_CONSERVATIVE_GC | TEST_CONSERVATIVE_GC
+        InterpreterLock lock;
+#endif
         _imp = i;
 #if !USE_CONSERVATIVE_GC
         _imp->ref();
-
 #endif
 #if USE_CONSERVATIVE_GC | TEST_CONSERVATIVE_GC
-	gcProtect(_imp);
+        gcProtect(_imp);
 #endif
     }
     
-    KJS::ObjectImp *rootObjectImp() const { return _imp; }
+    ObjectImp *rootObjectImp() const { return _imp; }
     
-    void setInterpreter (KJS::Interpreter *i);
-    KJS::Interpreter *interpreter() const { return _interpreter; }
+    void setInterpreter (Interpreter *i);
+    Interpreter *interpreter() const { return _interpreter; }
 
     void removeAllNativeReferences ();
 
@@ -91,8 +95,8 @@ public:
 
 private:
     const void *_nativeHandle;
-    KJS::ObjectImp *_imp;
-    KJS::Interpreter *_interpreter;
+    ObjectImp *_imp;
+    Interpreter *_interpreter;
 
     static FindRootObjectForNativeHandleFunctionPtr _findRootObjectForNativeHandleFunctionPtr;
     static CFRunLoopRef _runLoop;
