@@ -132,7 +132,7 @@ CString &CString::operator=(const CString &str)
   return *this;
 }
 
-bool KJS::operator==(const KJS::CString& c1, const KJS::CString& c2)
+bool operator==(const CString& c1, const CString& c2)
 {
   int len = c1.size();
   return len == c2.size() && (len == 0 || memcmp(c1.c_str(), c2.c_str(), len) == 0);
@@ -1138,7 +1138,7 @@ void UString::detach()
   }
 }
 
-bool KJS::operator==(const UString& s1, const UString& s2)
+bool operator==(const UString& s1, const UString& s2)
 {
   if (s1.rep->len != s2.rep->len)
     return false;
@@ -1147,7 +1147,7 @@ bool KJS::operator==(const UString& s1, const UString& s2)
 		 s1.rep->len * sizeof(UChar)) == 0);
 }
 
-bool KJS::operator==(const UString& s1, const char *s2)
+bool operator==(const UString& s1, const char *s2)
 {
   if (s2 == 0) {
     return s1.isEmpty();
@@ -1165,7 +1165,7 @@ bool KJS::operator==(const UString& s1, const char *s2)
   return u == uend && *s2 == 0;
 }
 
-bool KJS::operator<(const UString& s1, const UString& s2)
+bool operator<(const UString& s1, const UString& s2)
 {
   const int l1 = s1.size();
   const int l2 = s2.size();
@@ -1184,7 +1184,7 @@ bool KJS::operator<(const UString& s1, const UString& s2)
   return (l1 < l2);
 }
 
-int KJS::compare(const UString& s1, const UString& s2)
+int compare(const UString& s1, const UString& s2)
 {
   const int l1 = s1.size();
   const int l2 = s2.size();
@@ -1327,7 +1327,7 @@ CString UString::UTF8String() const
     } else if (c < 0x800) {
       *p++ = (char)((c >> 6) | 0xC0); // C0 is the 2-byte flag for UTF-8
       *p++ = (char)((c | 0x80) & 0xBF); // next 6 bits, with high bit set
-    } else if (c >= 0xD800 && c <= 0xDBFF && i < length && d[i+1].uc >= 0xDC00 && d[i+1].uc <= 0xDFFF) {
+    } else if (c >= 0xD800 && c <= 0xDBFF && i < length && d[i+1].uc >= 0xDC00 && d[i+2].uc <= 0xDFFF) {
       unsigned sc = 0x10000 + (((c & 0x3FF) << 10) | (d[i+1].uc & 0x3FF));
       *p++ = (char)((sc >> 18) | 0xF0); // F0 is the 4-byte flag for UTF-8
       *p++ = (char)(((sc >> 12) | 0x80) & 0xBF); // next 6 bits, with high bit set
