@@ -20,21 +20,26 @@
  * 
  */
 /**
-    File Name:          15.9.5.6.js
-    ECMA Section:       15.9.5.6
-    Description:        Date.prototype.getFullYear
+    File Name:          15.9.5.5.js
+    ECMA Section:       15.9.5.5
+    Description:        Date.prototype.getYear
+
+    This function is specified here for backwards compatibility only. The
+    function getFullYear is much to be preferred for nearly all purposes,
+    because it avoids the "year 2000 problem."
+
+        1.  Let t be this time value.
+    2.  If t is NaN, return NaN.
+    3.  Return YearFromTime(LocalTime(t)) 1900.
 
-   1.   Let t be this time value.
-   2.   If t is NaN, return NaN.
-   3.   Return YearFromTime(LocalTime(t)).
     Author:             christine@netscape.com
     Date:               12 november 1997
 */
 
-    var SECTION = "15.9.5.6";
+    var SECTION = "15.9.5.5";
     var VERSION = "ECMA_1";
     startTest();
-    var TITLE   = "Date.prototype.getFullYear()";
+    var TITLE   = "Date.prototype.getYear()";
 
     writeHeaderToLog( SECTION + " "+ TITLE);
 
@@ -53,55 +58,59 @@
 
     var UTC_FEB_29_2000 = TIME_2000 + 31*msPerDay + 28*msPerHour;
 
-    // get time for 1 jan 2005
-
-    var UTC_JAN_1_2005 = TIME_2000 + TimeInYear(2000)+TimeInYear(2001)+
-    TimeInYear(2002)+TimeInYear(2003)+TimeInYear(2004);
-
     addTestCase( now );
     addTestCase( time );
     addTestCase( TIME_1970 );
     addTestCase( TIME_1900 );
     addTestCase( TIME_2000 );
     addTestCase( UTC_FEB_29_2000 );
-    addTestCase( UTC_JAN_1_2005 );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date(NaN)).getFullYear()",
+                                    "(new Date(NaN)).getYear()",
                                     NaN,
-                                    (new Date(NaN)).getFullYear() );
+                                    (new Date(NaN)).getYear() );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "Date.prototype.getFullYear.length",
+                                    "Date.prototype.getYear.length",
                                     0,
-                                    Date.prototype.getFullYear.length );
+                                    Date.prototype.getYear.length );
 
     test();
 function addTestCase( t ) {
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date("+t+")).getFullYear()",
-                                    YearFromTime(LocalTime(t)),
-                                    (new Date(t)).getFullYear() );
+                                    "(new Date("+t+")).getYear()",
+                                    GetYear(YearFromTime(LocalTime(t))),
+                                    (new Date(t)).getYear() );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date("+(t+1)+")).getFullYear()",
-                                    YearFromTime(LocalTime(t+1)),
-                                    (new Date(t+1)).getFullYear() );
+                                    "(new Date("+(t+1)+")).getYear()",
+                                    GetYear(YearFromTime(LocalTime(t+1))),
+                                    (new Date(t+1)).getYear() );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date("+(t-1)+")).getFullYear()",
-                                    YearFromTime(LocalTime(t-1)),
-                                    (new Date(t-1)).getFullYear() );
+                                    "(new Date("+(t-1)+")).getYear()",
+                                    GetYear(YearFromTime(LocalTime(t-1))),
+                                    (new Date(t-1)).getYear() );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date("+(t-TZ_ADJUST)+")).getFullYear()",
-                                    YearFromTime(LocalTime(t-TZ_ADJUST)),
-                                    (new Date(t-TZ_ADJUST)).getFullYear() );
+                                    "(new Date("+(t-TZ_ADJUST)+")).getYear()",
+                                    GetYear(YearFromTime(LocalTime(t-TZ_ADJUST))),
+                                    (new Date(t-TZ_ADJUST)).getYear() );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date("+(t+TZ_ADJUST)+")).getFullYear()",
-                                    YearFromTime(LocalTime(t+TZ_ADJUST)),
-                                    (new Date(t+TZ_ADJUST)).getFullYear() );
+                                    "(new Date("+(t+TZ_ADJUST)+")).getYear()",
+                                    GetYear(YearFromTime(LocalTime(t+TZ_ADJUST))),
+                                    (new Date(t+TZ_ADJUST)).getYear() );
+}
+function GetYear( year ) {
+/*
+    if ( year >= 1900 && year < 2000 ) {
+        return year - 1900;
+    } else {
+        return year;
+    }
+*/
+    return year - 1900;
 }
 function test() {
     for ( tc=0; tc < testcases.length; tc++ ) {
